/********************************************************************
* Description: 4pod.c
*  kinematics for a square. looking at the square from above

bottom left corner: joint 0, x=0,y=0 "A"
bottom right corner: joint 1, x=bx,y=0 "B"
top left corner: joint 2, x = 0, y = cy "C"
top right corner: joint 3, x = bx, y = cy "D"

the math builds on the docs, and the syntax is about the same.  http://www.linuxcnc.org/docs/2.5/html/motion/kinematics.html

just for now, I am  10000 for all axis so  I don't run into limits while I play around. These should probably be set somewhere else. 
cy = 10000
by = 10000
********************************************************************/

#include "kinematics.h"
#include "posemath.h"
#include "hal.h"
#include "rtapi.h"
#include "rtapi_math.h"



int kinematicsForward(const double *joints,
		      EmcPose * pos,
		      const KINEMATICS_FORWARD_FLAGS * fflags,
		      KINEMATICS_INVERSE_FLAGS * iflags)
{
    double ap2 = joints[0]*joints[0];
	double bp2 = joints[1]*joints[1];
	double bx = 10000;
	double px = (ap2-bp2+bx*bx)/(2*bx);
	double py2 = ap2-px*px;
	if(py2<0) return -1;  
	
    pos->tran.x = px;
    pos->tran.y = sqrt(py2);
	return (0);

}
int kinematicsInverse(const EmcPose * pos,
		      double *joints,
		      const KINEMATICS_INVERSE_FLAGS * iflags,
		      KINEMATICS_FORWARD_FLAGS * fflags)
{       
    double px2 = pos->tran.x * pos->tran.x;
	double py2 = pos->tran.y * pos->tran.y;
    double bx = 10000;
	double cy = 10000;
	double bxmpx = (bx-pos->tran.x)*(bx-pos->tran.x);
	double cympy = (cy-pos->tran.y)*(cy-pos->tran.y);


    joints[0] = sqrt( px2 + py2 );
    joints[1] = sqrt( bxmpx + py2);
    joints[2] = sqrt( px2 + cympy );
    joints[3] = sqrt( bxmpx + cympy );
    return 0;
}

KINEMATICS_TYPE kinematicsType()
{
    return KINEMATICS_BOTH;
}

#include "rtapi.h"		/* RTAPI realtime OS API */
#include "rtapi_app.h"		/* RTAPI realtime module decls */
#include "hal.h"

EXPORT_SYMBOL(kinematicsType);
EXPORT_SYMBOL(kinematicsInverse);
EXPORT_SYMBOL(kinematicsForward);
MODULE_LICENSE("GPL");

int comp_id;
int rtapi_app_main(void) {
    comp_id = hal_init("4pod");
    if(comp_id < 0) return comp_id;
    hal_ready(comp_id);
    return 0;
}

void rtapi_app_exit(void) { hal_exit(comp_id); }
